@@ -339,7 +339,8 @@ setrange(bitarrayobject *self, Py_ssize_t start, Py_ssize_t stop, int val)
     }
 }
 
-/* Return number of 1 bits.  This function never fails. */
+/* Return number of 'vi' bits in range(start, stop).
+   This function never fails. */
 static Py_ssize_t
 count(bitarrayobject *self, int vi, Py_ssize_t start, Py_ssize_t stop)
 {
@@ -537,11 +538,9 @@ extend_bytes01(bitarrayobject *self, PyObject *bytes)
         case '\v':
             continue;
         default:
-            /* ensure no bits are added on error */
-            if (resize(self, original_nbits) < 0)
-                return -1;
             PyErr_Format(PyExc_ValueError, "expected '0' or '1' "
                          "(or whitespace), got '%c' (0x%02x)", c, c);
+            resize(self, original_nbits);  /* no bits added on error */
             return -1;
         }
         if (resize(self, self->nbits + 1) < 0)
@@ -633,44 +632,6 @@ unpack(bitarrayobject *self, char zero, char one, const char *fmt)
 
 /* --------- helper functions not involving bitarrayobjects ------------ */
 
-#ifdef IS_PY3K
-#define IS_INT_OR_BOOL(x)  (PyBool_Check(x) || PyLong_Check(x))
-#else  /* Py 2 */
-#define IS_INT_OR_BOOL(x)  (PyBool_Check(x) || PyInt_Check(x) || \
-                            PyLong_Check(x))
-#endif
-
-/* given a PyLong (which must be 0 or 1) or a PyBool, return 0 or 1,
-   or -1 on error */
-static int
-IntBool_AsInt(PyObject *v)
-{
-    long x;
-
-    if (PyBool_Check(v))
-        return v == Py_True;
-
-#if PY_MAJOR_VERSION == 2
-    if (PyInt_Check(v)) {
-        x = PyInt_AsLong(v);
-    }
-    else
-#endif
-    if (PyLong_Check(v)) {
-        x = PyLong_AsLong(v);
-    }
-    else {
-        PyErr_SetString(PyExc_TypeError, "int or bool expected");
-        return -1;
-    }
-
-    if (x < 0 || x > 1) {
-        PyErr_SetString(PyExc_ValueError, "int 0 or 1 expected");
-        return -1;
-    }
-    return (int) x;
-}
-
 /* Normalize index (which may be negative), such that 0 <= i <= n */
 static void
 normalize_index(Py_ssize_t n, Py_ssize_t *i)
@@ -909,7 +870,8 @@ bitarray_all(bitarrayobject *self)
 PyDoc_STRVAR(all_doc,
 "all() -> bool\n\
 \n\
-Return True when all bits in the array are True.");
+Return True when all bits in the array are True.\n\
+Note that `a.all()` is faster than `all(a)`.");
 
 
 static PyObject *
@@ -921,7 +883,8 @@ bitarray_any(bitarrayobject *self)
 PyDoc_STRVAR(any_doc,
 "any() -> bool\n\
 \n\
-Return True when any bit in the array is True.");
+Return True when any bit in the array is True.\n\
+Note that `a.any()` is faster than `any(a)`.");
 
 
 static PyObject *
@@ -1598,10 +1561,10 @@ bitarray_ass_item(bitarrayobject *self, Py_ssize_t i, PyObject *value)
 static int
 bitarray_contains(bitarrayobject *self, PyObject *item)
 {
-    if (IS_INT_OR_BOOL(item)) {
+    if (PyIndex_Check(item)) {
         int vi;
 
-        vi = IntBool_AsInt(item);
+        vi = IntOrBool_AsInt(item);
         if (vi < 0)
             return -1;
         return findfirst(self, vi, 0, self->nbits) >= 0;
@@ -1755,12 +1718,12 @@ setslice_bool(bitarrayobject *self, PyObject *slice, PyObject *bool)
     Py_ssize_t start, stop, step, slicelength, i, j;
     int vi;
 
-    assert(PySlice_Check(slice) && IS_INT_OR_BOOL(bool));
+    assert(PySlice_Check(slice) && PyIndex_Check(bool));
     if (PySlice_GetIndicesEx(slice, self->nbits,
                              &start, &stop, &step, &slicelength) < 0)
         return -1;
 
-    vi = IntBool_AsInt(bool);
+    vi = IntOrBool_AsInt(bool);
     if (vi < 0)
         return -1;
 
@@ -1833,7 +1796,7 @@ bitarray_ass_subscr(bitarrayobject *self, PyObject* item, PyObject* value)
         if (bitarray_Check(value))
             return setslice_bitarray(self, item, value);
 
-        if (IS_INT_OR_BOOL(value))
+        if (PyIndex_Check(value))
             return setslice_bool(self, item, value);
 
         PyErr_Format(PyExc_TypeError,
@@ -1876,7 +1839,7 @@ enum op_type {
 };
 
 /* perform bitwise in-place operation */
-static int
+static void
 bitwise(bitarrayobject *self, bitarrayobject *other, enum op_type oper)
 {
     const Py_ssize_t nbytes = Py_SIZE(self);
@@ -1898,9 +1861,8 @@ bitwise(bitarrayobject *self, bitarrayobject *other, enum op_type oper)
             self->ob_item[i] ^= other->ob_item[i];
         break;
     default:                    /* cannot happen */
-        return -1;
+        Py_FatalError("unknown bitwise operation");
     }
-    return 0;
 }
 
 /* return 0 if both a and b are bitarray objects, -1 on error */
@@ -1930,23 +1892,20 @@ bitwise_check(PyObject *a, PyObject *b, const char *ostr)
     return 0;
 }
 
-#define BITWISE_FUNC(oper, ostr)                                    \
-static PyObject *                                                   \
-bitarray_ ## oper (PyObject *self, PyObject *other)                 \
-{                                                                   \
-    PyObject *res;                                                  \
-                                                                    \
-    if (bitwise_check(self, other, ostr) < 0)                       \
-        return NULL;                                                \
-    res = bitarray_copy((bitarrayobject *) self);                   \
-    if (res == NULL)                                                \
-        return NULL;                                                \
-    if (bitwise((bitarrayobject *) res,                             \
-                (bitarrayobject *) other, OP_ ## oper) < 0) {       \
-        Py_DECREF(res);                                             \
-        return NULL;                                                \
-    }                                                               \
-    return res;                                                     \
+#define BITWISE_FUNC(oper, ostr)                             \
+static PyObject *                                            \
+bitarray_ ## oper (PyObject *self, PyObject *other)          \
+{                                                            \
+    PyObject *res;                                           \
+                                                             \
+    if (bitwise_check(self, other, ostr) < 0)                \
+        return NULL;                                         \
+    res = bitarray_copy((bitarrayobject *) self);            \
+    if (res == NULL)                                         \
+        return NULL;                                         \
+    bitwise((bitarrayobject *) res,                          \
+            (bitarrayobject *) other, OP_ ## oper);          \
+    return res;                                              \
 }
 
 BITWISE_FUNC(and, "&")               /* bitarray_and */
@@ -1960,11 +1919,10 @@ bitarray_i ## oper (PyObject *self, PyObject *other)         \
 {                                                            \
     if (bitwise_check(self, other, ostr) < 0)                \
         return NULL;                                         \
-    if (bitwise((bitarrayobject *) self,                     \
-                (bitarrayobject *) other, OP_ ## oper) < 0)  \
-        return NULL;                                         \
+    bitwise((bitarrayobject *) self,                         \
+            (bitarrayobject *) other, OP_ ## oper);          \
     Py_INCREF(self);                                         \
-    return (PyObject *) self;                                \
+    return self;                                             \
 }
 
 BITWISE_IFUNC(and, "&=")             /* bitarray_iand */
@@ -1972,55 +1930,133 @@ BITWISE_IFUNC(or,  "|=")             /* bitarray_ior  */
 BITWISE_IFUNC(xor, "^=")             /* bitarray_ixor */
 
 
+/* shift bitarray n positions to left (right=0) or right (right=1) */
+static void
+shift(bitarrayobject *a, Py_ssize_t n, int right)
+{
+    Py_ssize_t nbits = a->nbits;
+
+    if (n == 0)
+        return;
+
+    if (n >= nbits) {
+        memset(a->ob_item, 0x00, (size_t) Py_SIZE(a));
+        return;
+    }
+
+    assert(0 < n && n < nbits);
+    if (right) {                /* rshift */
+        copy_n(a, n, a, 0, nbits - n);
+        setrange(a, 0, n, 0);
+    }
+    else {                      /* lshift */
+        copy_n(a, 0, a, n, nbits - n);
+        setrange(a, nbits - n, nbits, 0);
+    }
+}
+
+/* check the shift bytes and return the shift count, -1 on error */
+static Py_ssize_t
+shift_check(PyObject *a, PyObject *b, const char *ostr)
+{
+    Py_ssize_t n;
+
+    if (!bitarray_Check(a) || !PyIndex_Check(b)) {
+        PyErr_Format(PyExc_TypeError,
+                     "unsupported operand type(s) for %s: '%s' and '%s'",
+                     ostr, Py_TYPE(a)->tp_name, Py_TYPE(b)->tp_name);
+        return -1;
+    }
+    n = PyNumber_AsSsize_t(b, PyExc_OverflowError);
+    if (n == -1 && PyErr_Occurred())
+        return -1;
+
+    if (n < 0) {
+        PyErr_SetString(PyExc_ValueError, "negative shift count");
+        return -1;
+    }
+    return n;
+}
+
+#define SHIFT_FUNC(name, inplace, right, ostr)         \
+static PyObject *                                      \
+bitarray_ ## name (PyObject *self, PyObject *other)    \
+{                                                      \
+    PyObject *res;                                     \
+    Py_ssize_t n;                                      \
+                                                       \
+    n = shift_check(self, other, ostr);                \
+    if (n < 0)                                         \
+        return NULL;                                   \
+    if (inplace) {                                     \
+        res = self;                                    \
+    }                                                  \
+    else {                                             \
+        res = bitarray_copy((bitarrayobject *) self);  \
+        if (res == NULL)                               \
+            return NULL;                               \
+    }                                                  \
+    shift((bitarrayobject *) res, n, right);           \
+    if (inplace)                                       \
+        Py_INCREF(res);                                \
+    return res;                                        \
+}
+
+SHIFT_FUNC(lshift,  0, 0, "<<")  /* bitarray_lshift */
+SHIFT_FUNC(rshift,  0, 1, ">>")  /* bitarray_rshift */
+SHIFT_FUNC(ilshift, 1, 0, "<<=") /* bitarray_ilshift */
+SHIFT_FUNC(irshift, 1, 1, ">>=") /* bitarray_irshift */
+
+
 static PyNumberMethods bitarray_as_number = {
-    0,                          /* nb_add */
-    0,                          /* nb_subtract */
-    0,                          /* nb_multiply */
+    0,                                   /* nb_add */
+    0,                                   /* nb_subtract */
+    0,                                   /* nb_multiply */
 #if PY_MAJOR_VERSION == 2
-    0,                          /* nb_divide */
+    0,                                   /* nb_divide */
 #endif
-    0,                          /* nb_remainder */
-    0,                          /* nb_divmod */
-    0,                          /* nb_power */
-    0,                          /* nb_negative */
-    0,                          /* nb_positive */
-    0,                          /* nb_absolute */
-    0,                          /* nb_bool (was nb_nonzero) */
-    (unaryfunc) bitarray_cpinvert,  /* nb_invert */
-    0,                          /* nb_lshift */
-    0,                          /* nb_rshift */
-    (binaryfunc) bitarray_and,  /* nb_and */
-    (binaryfunc) bitarray_xor,  /* nb_xor */
-    (binaryfunc) bitarray_or,   /* nb_or */
+    0,                                   /* nb_remainder */
+    0,                                   /* nb_divmod */
+    0,                                   /* nb_power */
+    0,                                   /* nb_negative */
+    0,                                   /* nb_positive */
+    0,                                   /* nb_absolute */
+    0,                                   /* nb_bool (was nb_nonzero) */
+    (unaryfunc) bitarray_cpinvert,       /* nb_invert */
+    (binaryfunc) bitarray_lshift,        /* nb_lshift */
+    (binaryfunc) bitarray_rshift,        /* nb_rshift */
+    (binaryfunc) bitarray_and,           /* nb_and */
+    (binaryfunc) bitarray_xor,           /* nb_xor */
+    (binaryfunc) bitarray_or,            /* nb_or */
 #if PY_MAJOR_VERSION == 2
-    0,                          /* nb_coerce */
+    0,                                   /* nb_coerce */
 #endif
-    0,                          /* nb_int */
-    0,                          /* nb_reserved (was nb_long) */
-    0,                          /* nb_float */
+    0,                                   /* nb_int */
+    0,                                   /* nb_reserved (was nb_long) */
+    0,                                   /* nb_float */
 #if PY_MAJOR_VERSION == 2
-    0,                          /* nb_oct */
-    0,                          /* nb_hex */
+    0,                                   /* nb_oct */
+    0,                                   /* nb_hex */
 #endif
-    0,                          /* nb_inplace_add */
-    0,                          /* nb_inplace_subtract */
-    0,                          /* nb_inplace_multiply */
+    0,                                   /* nb_inplace_add */
+    0,                                   /* nb_inplace_subtract */
+    0,                                   /* nb_inplace_multiply */
 #if PY_MAJOR_VERSION == 2
-    0,                          /* nb_inplace_divide */
+    0,                                   /* nb_inplace_divide */
 #endif
-    0,                          /* nb_inplace_remainder */
-    0,                          /* nb_inplace_power */
-    0,                          /* nb_inplace_lshift */
-    0,                          /* nb_inplace_rshift */
-    (binaryfunc) bitarray_iand, /* nb_inplace_and */
-    (binaryfunc) bitarray_ixor, /* nb_inplace_xor */
-    (binaryfunc) bitarray_ior,  /* nb_inplace_or */
-    0,                          /* nb_floor_divide */
-    0,                          /* nb_true_divide */
-    0,                          /* nb_inplace_floor_divide */
-    0,                          /* nb_inplace_true_divide */
+    0,                                   /* nb_inplace_remainder */
+    0,                                   /* nb_inplace_power */
+    (binaryfunc) bitarray_ilshift,       /* nb_inplace_lshift */
+    (binaryfunc) bitarray_irshift,       /* nb_inplace_rshift */
+    (binaryfunc) bitarray_iand,          /* nb_inplace_and */
+    (binaryfunc) bitarray_ixor,          /* nb_inplace_xor */
+    (binaryfunc) bitarray_ior,           /* nb_inplace_or */
+    0,                                   /* nb_floor_divide */
+    0,                                   /* nb_true_divide */
+    0,                                   /* nb_inplace_floor_divide */
+    0,                                   /* nb_inplace_true_divide */
 #if PY_MAJOR_VERSION == 3
-    0,                          /* nb_index */
+    0,                                   /* nb_index */
 #endif
 };
 
@@ -2194,11 +2230,11 @@ binode_make_tree(PyObject *codedict)
 
     while (PyDict_Next(codedict, &pos, &symbol, &value)) {
         if (check_value(value) < 0 ||
-            binode_insert_symbol(tree, (bitarrayobject *) value,
-                                 symbol) < 0) {
-            binode_delete(tree);
-            return NULL;
-        }
+            binode_insert_symbol(tree, (bitarrayobject *) value, symbol) < 0)
+            {
+                binode_delete(tree);
+                return NULL;
+            }
     }
     /* as we require the codedict to be non-empty the tree cannot be empty */
     assert(tree);
@@ -2838,7 +2874,7 @@ bitarray_from_index(PyTypeObject *type, PyObject *index, int endian)
     Py_ssize_t nbits;
 
     assert(PyIndex_Check(index));
-    nbits = PyNumber_AsSsize_t(index, PyExc_IndexError);
+    nbits = PyNumber_AsSsize_t(index, PyExc_OverflowError);
     if (nbits == -1 && PyErr_Occurred())
         return NULL;
 
@@ -3159,23 +3195,18 @@ the optional initial object, and endianness.\n\
 The initializer may be of the following types:\n\
 \n\
 `int`: Create a bitarray of given integer length.  The initial values are\n\
-arbitrary.  If you want all values to be set, use the .setall() method.\n\
+uninitialized.\n\
 \n\
 `str`: Create bitarray from a string of `0` and `1`.\n\
 \n\
-`list`, `tuple`, `iterable`: Create bitarray from a sequence, each\n\
-element in the sequence is converted to a bit using its truth value.\n\
-\n\
-`bitarray`: Create bitarray from another bitarray.  This is done by\n\
-copying the buffer holding the bitarray data, and is hence very fast.\n\
+`iterable`: Create bitarray from a sequence, each element in the sequence\n\
+is converted to a bit using its truth value.\n\
 \n\
 The optional keyword arguments `endian` specifies the bit endianness of the\n\
 created bitarray object.\n\
 Allowed values are the strings `big` and `little` (default is `big`).\n\
-\n\
-Note that setting the bit endianness only has an effect when accessing the\n\
-machine representation of the bitarray, i.e. when using the methods: tofile,\n\
-fromfile, tobytes, frombytes.");
+The bit endianness only effects the when buffer representation of the\n\
+bitarray.");
 
 static PyTypeObject Bitarray_Type = {
 #ifdef IS_PY3K
@@ -3205,7 +3236,7 @@ static PyTypeObject Bitarray_Type = {
     &bitarray_as_buffer,                      /* tp_as_buffer */
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE | Py_TPFLAGS_HAVE_WEAKREFS
 #if PY_MAJOR_VERSION == 2
-    | Py_TPFLAGS_HAVE_NEWBUFFER
+    | Py_TPFLAGS_HAVE_NEWBUFFER | Py_TPFLAGS_CHECKTYPES
 #endif
     ,                                         /* tp_flags */
     bitarraytype_doc,                         /* tp_doc */
